/* 
 * MBDyn (C) is a multibody analysis code. 
 * http://www.mbdyn.org
 *
 * Copyright (C) 1996-2000
 *
 * Pierangelo Masarati	<masarati@aero.polimi.it>
 * Paolo Mantegazza	<mantegazza@aero.polimi.it>
 *
 * Dipartimento di Ingegneria Aerospaziale - Politecnico di Milano
 * via La Masa, 34 - 20156 Milano, Italy
 * http://www.aero.polimi.it
 *
 * Changing this copyright notice is forbidden.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation (version 2 of the License).
 * 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include <mbconfig.h>           /* This goes first in every *.c,*.cc file */
#endif /* HAVE_CONFIG_H */

#ifdef HAVE_LOADABLE

#include <dlfcn.h>
#include <unistd.h>

#include <loadable.h>
#include <dataman.h>

/* funzioni di default */
unsigned int 
__i_get_num_dof(const LoadableElem* /* pEl */ )
{
   	return 0;
}

DofOrder::Order 
__set_dof(const LoadableElem*, unsigned int /* i */ )
{
   	cerr << "You shouldn't be here!" << endl;
   	THROW(ErrGeneric());
#ifndef USE_EXCEPTIONS
   	return DofOrder::UNKNOWN;
#endif /* USE_EXCEPTIONS */
}

void
__output(const LoadableElem* /* pEl */ , OutputHandler& /* OH */ )
{
   	NO_OP;
}

ostream& 
__restart(const LoadableElem* pEl , ostream& out)
{
   	return out << "loadable: " << pEl->GetLabel() 
		<< ", not implemented yet;" << endl;
}

void 
__work_space_dim(const LoadableElem* /* pEl */ ,
		 integer* piNumRows, 
		 integer* piNumCols)
{
   	*piNumRows = 0;
   	*piNumCols = 0;
}

VariableSubMatrixHandler& 
__ass_jac(LoadableElem* /* pEl */ ,
	  VariableSubMatrixHandler& WorkMat,
	  doublereal /* dCoef */ ,
	  const VectorHandler& /* XCurr */ ,
	  const VectorHandler& /* XPrimeCurr */ )
{
   	WorkMat.SetNullMatrix();
   	return WorkMat;
}

void 
__ass_eig(LoadableElem* /* pEl */ ,
	  VariableSubMatrixHandler& WorkMatA,
	  VariableSubMatrixHandler& WorkMatB,
	  const VectorHandler& /* XCurr */ ,
	  const VectorHandler& /* XPrimeCurr */ )
{
   	WorkMatA.SetNullMatrix();
   	WorkMatB.SetNullMatrix(); 
}

SubVectorHandler& 
__ass_res(LoadableElem* /* pEl */ ,
	  SubVectorHandler& WorkVec,
	  doublereal /* dCoef */ ,
	  const VectorHandler& /* XCurr */ ,
	  const VectorHandler& /* XPrimeCurr */ )
{
   	WorkVec.Resize(0);
   	return WorkVec;
}

void 
__before_predict(const LoadableElem* /* pEl */ ,
		 VectorHandler& /* X */ ,
		 VectorHandler& /* XP */ ,
		 VectorHandler& /* XPrev */ ,
		 VectorHandler& /* XPPrev */ )
{
   	NO_OP;
}

void 
__after_predict(const LoadableElem* /* pEl */ ,
		VectorHandler& /* X */ ,
		VectorHandler& /* XP */ )
{
   	NO_OP;
}

void 
__update(LoadableElem* /* pEl */ ,
	 const VectorHandler& /* X */ ,
	 const VectorHandler& /* XP */ )
{
   	NO_OP;
}

unsigned int 
__i_get_initial_num_dof(const LoadableElem* /* pEl */ )
{
   	return 0;
}

void 
__initial_work_space_dim(const LoadableElem* /* pEl */ ,
			 integer* piNumRows, 
			 integer* piNumCols)
{
   	*piNumRows = 0;
   	*piNumCols = 0;   
}

VariableSubMatrixHandler& 
__initial_ass_jac(LoadableElem* /* pEl */ ,
		  VariableSubMatrixHandler& WorkMat, 
		  const VectorHandler& /* XCurr */ )
{
   	WorkMat.SetNullMatrix();
   	return WorkMat;
}

SubVectorHandler& 
__initial_ass_res(LoadableElem* /* pEl */ ,
		  SubVectorHandler& WorkVec, 
		  const VectorHandler& /* XCurr */ )
{  
   	WorkVec.Resize(0);
   	return WorkVec;
}

void 
__set_value(const LoadableElem* /* pEl */ , 
	    VectorHandler& /* X */ ,
	    VectorHandler& /* XP */ )
{
   	NO_OP;
}

void 
__set_initial_value(const LoadableElem* /* pEl */ , VectorHandler& /* X */ )
{
   	NO_OP;
}

unsigned int 
__i_get_num_priv_data(const LoadableElem* /* pEl */ )
{
   	return 0;
}

doublereal 
__d_get_priv_data(const LoadableElem* /* pEl */ , unsigned int /* i */ )
{
   	cerr << "You shouldn't be here!" << endl;
   	THROW(ErrGeneric());
#ifndef USE_EXCEPTIONS
        return 0.;
#endif /* USE_EXCEPTIONS */
}

void 
__destroy(LoadableElem* /* pEl */ )
{
   	NO_OP;
}

/* metodi della classe */

LoadableElem::LoadableElem(unsigned int uLabel, 
			   const DofOwner* pDO, 
			   DataManager* pDM, 
			   MBDynParser& HP)
: Elem(uLabel, Elem::LOADABLE, flag(0)),
#ifdef USE_STRUCT_NODES
InitialAssemblyElem(uLabel, Elem::LOADABLE, flag(0)),
#ifdef USE_AERODYNAMIC_ELEMS
AerodynamicElem(uLabel, AerodynamicElem::AERODYNAMICLOADABLE, flag(0)),
#endif /* USE_AERODYNAMIC_ELEMS */
ElemGravityOwner(uLabel, Elem::LOADABLE, flag(0)),
#endif /* USE_STRUCT_NODES */
ElemWithDofs(uLabel, Elem::LOADABLE, pDO, flag(0)),
priv_data(NULL),
module_name(NULL),
handle(NULL),
calls(NULL)
{
   	ASSERT(pDM != NULL);
   
   	/* nome del modulo */
   	const char* s = HP.GetFileName();
	if (s == NULL) {
		cerr << "Loadable(" << uLabel
			<< "): unable to get module name" << endl;
		THROW(ErrGeneric());
	}

   	SAFESTRDUP(module_name, s);
   	handle = dlopen(module_name, RTLD_NOW /* RTLD_LAZY */ );
   	if (handle == NULL) {
		/* look for module in cwd */
		size_t l = strlen(module_name);
		char cwd[1024];

		if (getcwd(cwd, 1024) == NULL || strlen(cwd) >= 1024-1-3-l) {
			cerr << "Loadable(" << uLabel 
				<< "): unable to get current working directory"
				<< endl;
	                THROW(ErrGeneric());
		}

		strcat(cwd, "/");
		strcat(cwd, module_name);
		if (strcmp(module_name+l-3, ".so") != 0) {
			strcat(cwd, ".so");
		}

		handle = dlopen(cwd, RTLD_NOW /* RTLD_LAZY */ );
		if (handle == NULL) {
      			cerr << "Loadable(" << uLabel 
				<< "): unable to open module <" << module_name 
				<< "> (dlopen returns \"" << dlerror() << "\")" 
				<< endl;
      			THROW(ErrGeneric());
		}
   	}

	const char *data_name = NULL;
	LoadableCalls **tmpcalls = NULL;
	if (HP.IsKeyWord("name")) {
		data_name = HP.GetStringWithDelims();
	} else {
		data_name = "calls";
	}
   	DEBUGCOUT("binding to data \"" << data_name
     		<< "\" (must be def'd!)" << endl);
	tmpcalls = (LoadableCalls **)dlsym(handle, data_name);
	
   	if (tmpcalls == NULL) {
      		const char* err = dlerror();
      		if (err == NULL) {
	 		cerr << "Loadable(" << uLabel 
	   			<< "): data \"" << data_name
	   			<< "\" must be defined in module <" 
				<< module_name << ">" << endl;
      		} else {
	 		cerr << "Loadable(" << uLabel
	   			<< "): error while binding to data \"" 
				<< data_name
	   			<< "\" in module <" << module_name
	   			<< "> (dlsym returns \"" << err 
				<< "\")" << endl;
      		}
      		THROW(ErrGeneric());
   	}

	calls = *tmpcalls;
   
	if (calls->read == NULL) {
		cerr << "Loadable(" << uLabel
			<< "): function \"read\" must be defined in module <"
			<< module_name << "> data" << endl;
		THROW(ErrGeneric());
	}

	/*
	 * Mette i default ove servono
	 */
	if (calls->i_get_num_dof == NULL) {
		calls->i_get_num_dof = __i_get_num_dof;
	}

	if (calls->set_dof == NULL) {
		calls->set_dof = __set_dof;
	}

	if (calls->output == NULL) {
		calls->output = __output;
	}

	if (calls->restart == NULL) {
		calls->restart = __restart;
	}

	if (calls->work_space_dim == NULL) {
		calls->work_space_dim = __work_space_dim;
	}

	if (calls->ass_jac == NULL) {
		calls->ass_jac = __ass_jac;
	}

	if (calls->ass_eig == NULL) {
		calls->ass_eig = __ass_eig;
	}

	if (calls->ass_res == NULL) {
		calls->ass_res = __ass_res;
	}

	if (calls->before_predict == NULL) {
		calls->before_predict = __before_predict;
	}

	if (calls->after_predict == NULL) {
		calls->after_predict = __after_predict;
	}

	if (calls->update == NULL) {
		calls->update = __update;
	}

	if (calls->i_get_initial_num_dof == NULL) {
		calls->i_get_initial_num_dof = __i_get_initial_num_dof;
	}

	if (calls->initial_work_space_dim == NULL) {
		calls->initial_work_space_dim = __initial_work_space_dim;
	}

	if (calls->initial_ass_jac == NULL) {
		calls->initial_ass_jac = __initial_ass_jac;
	}

	if (calls->initial_ass_res == NULL) {
		calls->initial_ass_res = __initial_ass_res;
	}

	if (calls->set_value == NULL) {
		calls->set_value = __set_value;
	}

	if (calls->set_initial_value == NULL) {
		calls->set_initial_value = __set_initial_value;
	}

	if (calls->i_get_num_priv_data == NULL) {
		calls->i_get_num_priv_data = __i_get_num_priv_data;
	}

	if (calls->d_get_priv_data == NULL) {
		calls->d_get_priv_data = __d_get_priv_data;
	}

	if (calls->destroy == NULL) {
		calls->destroy = __destroy;
	}
	
   	priv_data = (*calls->read)(this, pDM, HP, pDM->pGetDrvHdl());
   	SetOutputFlag(pDM->fReadOutput(HP, Elem::LOADABLE)); 
}

LoadableElem::~LoadableElem(void)
{
	ASSERT(calls->destroy != NULL);
   	(*calls->destroy)(this);
   
   	ASSERT(handle != NULL);
   	dlclose(handle);
   	SAFEDELETEARR(module_name);
}

Elem::Type 
LoadableElem::GetElemType(void) const
{
   	return Elem::LOADABLE;
}

unsigned int 
LoadableElem::iGetNumDof(void) const
{
	ASSERT(calls->i_get_num_dof != NULL);
   	return (*calls->i_get_num_dof)(this);
}

DofOrder::Order 
LoadableElem::SetDof(unsigned int i) const
{
   	ASSERT(i < iGetNumDof());
	ASSERT(calls->set_dof != NULL);
   	return (*calls->set_dof)(this, i);
}

void 
LoadableElem::Output(OutputHandler& OH) const
{
	ASSERT(calls->output != NULL);
   	(*calls->output)(this, OH);
}

ostream& 
LoadableElem::Restart(ostream& out) const
{
	ASSERT(calls->restart != NULL);
   	out << "    loadable: " << GetLabel() << ", \"" 
		<< module_name << "\", ";
   	return (*calls->restart)(this, out) << ';' << endl;
}

void 
LoadableElem::WorkSpaceDim(integer* piNumRows, integer* piNumCols) const
{
	ASSERT(calls->work_space_dim != NULL);
   	(*calls->work_space_dim)(this, piNumRows, piNumCols);
}

VariableSubMatrixHandler& 
LoadableElem::AssJac(VariableSubMatrixHandler& WorkMat,
		     doublereal dCoef, 
		     const VectorHandler& XCurr,
		     const VectorHandler& XPCurr)
{
	ASSERT(calls->ass_jac != NULL);
   	return (*calls->ass_jac)(this, WorkMat, dCoef, XCurr, XPCurr);
}

void
LoadableElem::AssEig(VariableSubMatrixHandler& WorkMatA,
		     VariableSubMatrixHandler& WorkMatB,
		     const VectorHandler& XCurr,
		     const VectorHandler& XPCurr)
{
   	ASSERT(calls->ass_eig != NULL);
   	(*calls->ass_eig)(this, WorkMatA, WorkMatB, XCurr, XPCurr);
}

SubVectorHandler& 
LoadableElem::AssRes(SubVectorHandler& WorkVec,
		     doublereal dCoef,
		     const VectorHandler& XCurr, 
		     const VectorHandler& XPCurr)
{
   	ASSERT(calls->ass_res != NULL);
   	return (*calls->ass_res)(this, WorkVec, 
					    dCoef, XCurr, XPCurr);
}

void 
LoadableElem::BeforePredict(VectorHandler& X,
			    VectorHandler& XP,
			    VectorHandler& XPrev,
			    VectorHandler& XPPrev) const
{
   	ASSERT(calls->before_predict != NULL);
   	(*calls->before_predict)(this, X, XP, XPrev, XPPrev);
}

void 
LoadableElem::AfterPredict(VectorHandler& X,
			   VectorHandler& XP)
{
   	ASSERT(calls->after_predict != NULL);
   	(*calls->after_predict)(this, X, XP);
}


void 
LoadableElem::Update(const VectorHandler& XCurr, 
		     const VectorHandler& XPrimeCurr)
{
   	ASSERT(calls->update != NULL);
   	(*calls->update)(this, XCurr, XPrimeCurr);
}

#ifdef USE_STRUCT_NODES
unsigned int 
LoadableElem::iGetInitialNumDof(void) const
{
   	ASSERT(calls->i_get_initial_num_dof != NULL);
   	return (*calls->i_get_initial_num_dof)(this);
}

void 
LoadableElem::InitialWorkSpaceDim(integer* piNumRows, integer* piNumCols) const
{
   	ASSERT(calls->initial_work_space_dim != NULL);
   	(*calls->initial_work_space_dim)(this, piNumRows, piNumCols);
}

VariableSubMatrixHandler& 
LoadableElem::InitialAssJac(VariableSubMatrixHandler& WorkMat,
			    const VectorHandler& XCurr)
{
   	ASSERT(calls->initial_ass_jac != NULL);
   	return (*calls->initial_ass_jac)(this, WorkMat, XCurr);
}

SubVectorHandler& 
LoadableElem::InitialAssRes(SubVectorHandler& WorkVec, 
			    const VectorHandler& XCurr)
{
   	ASSERT(calls->initial_ass_res != NULL);
   	return (*calls->initial_ass_res)(this, WorkVec, XCurr);
}

void 
LoadableElem::SetInitialValue(VectorHandler& X) const
{   
   	ASSERT(calls->set_initial_value != NULL);
   	(*calls->set_initial_value)(this, X);
}
#endif /* USE_STRUCT_NODES */

void 
LoadableElem::SetValue(VectorHandler& X, VectorHandler& XP) const
{
   	ASSERT(calls->set_value != NULL);
   	(*calls->set_value)(this, X, XP);
}

unsigned int 
LoadableElem::iGetNumPrivData(void) const
{
   	ASSERT(calls->i_get_num_priv_data != NULL);
   	return (*calls->i_get_num_priv_data)(this);
}

doublereal 
LoadableElem::dGetPrivData(unsigned int i) const
{
   	ASSERT(calls->d_get_priv_data != NULL);
   	return (*calls->d_get_priv_data)(this, i);
}

Elem* 
ReadLoadable(DataManager* pDM, 
	     MBDynParser& HP, 
	     const DofOwner* pDO, 
	     unsigned int uLabel)
{
   	Elem* pEl = NULL;
   
   	SAFENEWWITHCONSTRUCTOR(pEl,
			       LoadableElem,
			       LoadableElem(uLabel, pDO, pDM, HP));
   	return pEl;
}

#endif /* HAVE_LOADABLE */

